"""
Tiros Visualization Flask App
"""
import glob
import os
import sys
from flask import Flask, render_template, request, jsonify
from flask import redirect, url_for, session, escape

from tiros.tiros_viz.process_viz import TirosVizProc
from tiros.tiros_viz import utils
from tiros.tiros_viz import render_error
from tiros.tiros_viz import tiros_config
from tiros import server

TIROS_CONFIG = {'host': tiros_config.HOST,
                 'ssl': tiros_config.SSL,
                 'session': session}
app = Flask(__name__)
app.secret_key = os.urandom(24)
viz = TirosVizProc(False)
tiros_session = None


# Render main view
@app.route('/')
def index():
    return render_template('index.html')


# Route to render the uploaded snapshot
@app.route('/snapshot', methods=['POST'])
def snapshot():
    app.logger.info('Render Snapshot View')
    input_file = request.files.get("file")
    all_views = request.form['all_views'] if 'all_views' in request.form else None
    do_query = request.form['general_query'] if 'general_query' in request.form else None
    file_ok = utils.check_input_file(input_file, app)
    if not file_ok:
        return render_error.error_file()
    snap_dir, snap_path = utils.save_snap_file(input_file, app)
    viz_path, cached_result = viz.run(
        snap_path, snap_dir, do_query, TIROS_CONFIG)
    if not viz_path:
        return render_error.viz_error()
    cached_status, tiros_status = utils.check_error(cached_result, do_query)
    rel = os.path.relpath(viz_path)
    relative_viz_path = rel.split('tiros_viz/')[1]
    session['relative_viz_path'] = relative_viz_path
    session['current_view'] = os.path.basename(viz_path)
    set_session(snap_path, cached_status,
                tiros_status, snap_dir,
                relative_viz_path)
    return render_template('snapshot_view.html',
                           snapshot_file=relative_viz_path,
                           cached_status=cached_status,
                           tiros_status=tiros_status)


# Route that will process Tiros query
@app.route('/query_page', methods=['POST'])
def query_page():
    query = request.form['text']
    app.logger.info('Query: %s', query)
    response = viz.inline_tiros(session['snap_file'],
                                session['current_view'],
                                session['current_viz_data'],
                                session['snap_dir'],
                                TIROS_CONFIG,
                                query)
    session['current_view'] = os.path.basename(response['viz_json_path'])
    return render_template('query_view.html',
                           query_answer=response['message'],
                           snapshot_file=response['viz_json_path'],
                           cached_status=session['cached_status'],
                           tiros_status=session['tiros_status'])


# Route that will process different snapshot view (e.g. vsi, acl)
@app.route('/snap_view', methods=['POST'])
def snap_view():
    select = request.form.get('selection')
    app.logger.info("Snap view " + str(select))
    current_snap = session['current_viz_data']
    snap_dir = session['snap_dir']
    if select == "vsi":
        session['current_view'] = os.path.basename(current_snap)
        return render_template('snapshot_view.html',
                               cached_status=session['cached_status'],
                               tiros_status=session['tiros_status'],
                               snapshot_file=current_snap)
    view_path = utils.get_view_path(select, current_snap, snap_dir)
    session['current_view'] = os.path.basename(view_path)
    return render_template('snapshot_view.html',
                           cached_status=session['cached_status'],
                           tiros_status=session['tiros_status'],
                           snapshot_file=view_path)


# Route that will process cached result
@app.route('/cached_result', methods=['POST'])
def cached_result():
    select = request.form.get('selection')
    app.logger.info("Cached result: {}".format(select))
    current_viz = session['current_viz_data']
    snap_dir = session['snap_dir']
    snap_path = utils.get_view_path(select, current_viz, snap_dir)
    session['current_view'] = os.path.basename(snap_path)
    app.logger.info("Serving: {}".format(snap_path))
    return render_template('snapshot_view.html',
                           cached_status=session['cached_status'],
                           snap_status=session['tiros_status'],
                           tiros_status=session['tiros_status'],
                           snapshot_file=snap_path)


@app.after_request
def add_header(r):
    """
    Add headers to render in different browsers
    also no caching
    """
    r.headers["Cache-Control"] = "no-cache, no-store, must-revalidate"
    r.headers["Pragma"] = "no-cache"
    r.headers["Expires"] = "0"
    r.headers['Cache-Control'] = 'public, max-age=0'
    return r


def set_session(snap_file, cached_status, tiros_status, snap_dir, relative_path):
    """
    Set session data
    """
    session['snap_file'] = snap_file
    session['cached_status'] = cached_status
    session['tiros_status'] = tiros_status
    session['snap_dir'] = snap_dir
    session['current_viz_data'] = relative_path
    return

def wrap_args(session, args):
    ssl = not args.no_ssl
    host = args.host or (server.DEV_HOST if args.dev else server.PROD_HOST)
    return {'host': host,
            'ssl': ssl,
            'session': session}

def main(session, args):
    global TIROS_CONFIG
    TIROS_CONFIG = wrap_args(session, args)
    app.run(
        host="localhost",
        port=args.viz_port,
        debug=True
    )


if __name__ == '__main__':
    main()
